package sun.security.util.math.intpoly;

import java.nio.ByteBuffer;
import java.math.BigInteger;

public class IntegerPolynomial1305 extends IntegerPolynomial
{
    protected static final int SUBTRAHEND = 5;
    protected static final int NUM_LIMBS = 5;
    private static final int POWER = 130;
    private static final int BITS_PER_LIMB = 26;
    private static final BigInteger MODULUS;
    
    public IntegerPolynomial1305() {
        super(26, 5, 1, IntegerPolynomial1305.MODULUS);
    }
    
    @Override
    protected void mult(long[] array, long[] array2, long[] array3) {
        this.carryReduce(array3, array[0] * array2[0], array[0] * array2[1] + array[1] * array2[0], array[0] * array2[2] + array[1] * array2[1] + array[2] * array2[0], array[0] * array2[3] + array[1] * array2[2] + array[2] * array2[1] + array[3] * array2[0], array[0] * array2[4] + array[1] * array2[3] + array[2] * array2[2] + array[3] * array2[1] + array[4] * array2[0], array[1] * array2[4] + array[2] * array2[3] + array[3] * array2[2] + array[4] * array2[1], array[2] * array2[4] + array[3] * array2[3] + array[4] * array2[2], array[3] * array2[4] + array[4] * array2[3], array[4] * array2[4]);
    }
    
    private void carryReduce(long[] array, long n, long n2, long n3, long n4, long n5, long n6, long n7, long n8, long n9) {
        array[2] = n3 + n8 * 5L;
        n4 += n9 * 5L;
        long carryValue = this.carryValue(n4);
        array[3] = n4 - (carryValue << 26);
        n5 += carryValue;
        long carryValue2 = this.carryValue(n5);
        array[4] = n5 - (carryValue2 << 26);
        n6 += carryValue2;
        array[0] = n + n6 * 5L;
        array[1] = n2 + n7 * 5L;
        this.carry(array);
    }
    
    @Override
    protected void square(long[] array, long[] array2) {
        this.carryReduce(array2, array[0] * array[0], 2L * (array[0] * array[1]), 2L * (array[0] * array[2]) + array[1] * array[1], 2L * (array[0] * array[3] + array[1] * array[2]), 2L * (array[0] * array[4] + array[1] * array[3]) + array[2] * array[2], 2L * (array[1] * array[4] + array[2] * array[3]), 2L * (array[2] * array[4]) + array[3] * array[3], 2L * (array[3] * array[4]), array[4] * array[4]);
    }
    
    @Override
    protected void encode(ByteBuffer byteBuffer, int n, byte b, long[] array) {
        if (n == 16) {
            this.encode(byteBuffer.getLong(), byteBuffer.getLong(), b, array);
        }
        else {
            super.encode(byteBuffer, n, b, array);
        }
    }
    
    protected void encode(long n, long n2, byte b, long[] array) {
        array[0] = (n2 & 0x3FFFFFFL);
        array[1] = (n2 >>> 26 & 0x3FFFFFFL);
        array[2] = (n2 >>> 52) + ((n & 0x3FFFL) << 12);
        array[3] = (n >>> 14 & 0x3FFFFFFL);
        array[4] = (n >>> 40) + (long)(b << 24);
    }
    
    private void modReduceIn(long[] array, int n, long n2) {
        long n3 = n2 * 5L;
        int n4 = n - 5;
        array[n4] += n3;
    }
    
    @Override
    protected void finalCarryReduceLast(long[] array) {
        long n = array[this.numLimbs - 1] >> this.bitsPerLimb;
        int n2 = this.numLimbs - 1;
        array[n2] -= n << this.bitsPerLimb;
        this.modReduceIn(array, this.numLimbs, n);
    }
    
    protected final void modReduce(long[] array, int i, int n) {
        for (; i < n; array[i] = 0L, ++i) {
            this.modReduceIn(array, i, array[i]);
        }
    }
    
    protected void modReduce(long[] array) {
        this.modReduce(array, 5, 4);
    }
    
    @Override
    protected long carryValue(long n) {
        return n >> 26;
    }
    
    @Override
    protected void postEncodeCarry(long[] array) {
    }
    
    @Override
    protected void reduce(long[] array) {
        long n = this.carryOut(array, 3) + array[4];
        long carryValue = this.carryValue(n);
        array[4] = n - (carryValue << 26);
        this.modReduceIn(array, 5, carryValue);
        this.carry(array);
    }
    
    static {
        MODULUS = IntegerPolynomial1305.TWO.pow(130).subtract(BigInteger.valueOf(5L));
    }
}
